## get_osc.py
#
#  Get OSC data and decode
#
#############################################################################

import GameLogic as gl
import socket
from OSC3.OSC import decodeOSC
from time import sleep

scenes = gl.getSceneList()

def main():
    ip = "127.0.0.1"
    buffer_size = 1024
    port = gl.port_in

    connect_blender(ip, port, buffer_size)
    get_OSC_data(buffer_size)
    special_message()

def connect_blender(ip, port, buffer_size):
    # Connexion de Blender à Relay
    if not gl.connected :
        # Si le socket existe déjà
        try:
            gl.sock.close()
            print("Socket.close() in connect_blender in get_osc.py")
        except:
            pass

        gl.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            gl.sock.bind((ip, port))
            gl.sock.setblocking(0)
            gl.sock.settimeout(0.02)
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, buffer_size)
            print('Get OSC on IP={} Port={} Buffer={}'.format(ip, port, buffer_size))
            gl.connected = True
        except:
            print('No connected to IP={} Port={}'.format(ip, port),
                                        ", gl.connected =", gl.connected)
            pass

def get_OSC_data(buffer_size):
    # Si blender est connecté, récupération des messages OSC
    if gl.connected :
        try:
            raw_data = gl.sock.recv(buffer_size)
            gl.data = decodeOSC(raw_data)
            #print(gl.data)
        except socket.error:
            print("Socket Error: no data, no players_data")
            pass

def special_message():
    if '/ball_reset' in gl.data :
        print("OSC message /ball_reset received: balle au centre")
        ball_in_center()

    if '/quit' in gl.data :
        print("OSC message /quit received")
        quit_game()

    if '/general_reset' in gl.data :
        print("OSC message /general_reset received: new_game(), reset_variables()")
        new_game()

    if '/final_display' in gl.data :
        # Je valide la réception
        gl.final_display_received = 1
        # Affichage de la scène Rank
        print("OSC message /final_display received")
        if gl.level > 1:
            if gl.rank_scene != 2:
                gl.rank_scene = 1
        gl.rank_display_bug = 0
        gl.text_rank = 1

    if not '/final_display' in gl.data :
        gl.rank_display_bug += 1
        gl.text_rank = 0

def quit_game():
    #
    print("Un joueur a quitté le jeu")


def ball_in_center():
    # Balle au centre si reset avec R
    if gl.sendosc["reset"] == True:
        gl.ball.localPosition = [0, 0, 1]

def reset_score():
    # Reset des scores du niveau en cours
    for i in range(gl.level):
        gl.goal[i]["score"] = 9
    # Il manque la machine au niveau 1: goal[1] existe toujours !
    gl.goal[1]["score"] = 9

def reset_variables():
    # Reset de variables pour repartir à zéro
    reset_score()
    gl.rank_display_bug = 0
    gl.tempo_globale = 0

    # level 1 only
    gl.level1_rated = 0 # si classement du level 1 fait dans game.py
    gl.classement_level1 = {}
    gl.tempo_rank_level1 = 0

    # pour afficher le classement si j'ai perdu
    gl.rank_scene = 0

    # Le dictionnaire de classement des joueurs
    gl.classement = {}

    # blocage de la bat si j'ai fini
    gl.block = 0

def new_game():
    reset_variables()
    ### Supprime la scène précédente, ajoute la scene 1
    for scn in scenes :
        if "Rank" in scn.name:
            scn.end()
        if "_players" in scn.name:
            scn.end()

cont = gl.getCurrentController()
sens = cont.sensors["Get OSC"]
if sens.positive:
    main()
