#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

from twisted.internet import reactor
from txosc import async, dispatch, osc

from time import time, sleep

class GameManagement(object):
    ''' Game Management with all datas from all players
        OSC Message address:   message 10 values ['/ball', 8.7, 0, '/bat', -9.5, 0.0, '/score', 9, '/name', 'gffffg']


        manage ={ (   ip,       socket): ['name', classement,    ['/ball',   7,   6, '/bat', 9, 0, '/score', 5, '/name', 'moi']]}
                  key[0] key[1]           value[0] value[1]   value[2][0]   [1]  [2]   [3]   4  5       6     7    8       9

    '''
    def __init__(self):
        # Dict
        self.manage = {}
        # Connected players: ACTA[address] = time of the last connexion
        self.ACTA = {}
        # The winner if
        self.winner = False
        self.since = time()
        self.block = 0

    def insert_data_in_dict(self, message, address):
        # Called only with /player_data message
        # Update datas only if name != machine, sometimes machine is in the list !
        if message[9] != "machine":
            if address in self.manage:
                self.manage[address][2] = message
            else:
                # set value of this key in dict
                self.manage[address] = [message[9]] # Name
                self.manage[address].append(0) # Default classement
                self.manage[address].append(message) # datas
            #print "self.manage", self.manage

        # Update classement
        self.update_classement()

        # Update ACTA
        self.ACTA[address] = time()

    def update_classement(self):
        # update classement if a score is zero
        for key, value in self.manage.items():
            # if score = 0, this player loose
            if value[2][7] == 0: # score = 0
                # He is never classified ?
                if value[1] == 0:
                    # Find his classement
                    classified = []
                    for k, v in self.manage.items():
                        if v[1] != 0:
                            classified.append(v[1])
                    if classified:
                        classified.sort()
                        his_classement = classified[0] - 1
                    else:
                        his_classement = len(self.manage)
                    self.manage[key][1] = his_classement
        # but the last player win though his score is nether zero
        # if somebody is second, someone is first
        for value in self.manage.values():
            # someone is 2
            if value[1] == 2:
                for key, value in self.manage.items():
                    # someone is 0
                    if value[1] == 0:
                        # I find the winner
                        value[1] = 1

    def who_is_the_winner(self):
        for key, value in self.manage.items():
            if value[1] == 1:
                self.winner = value[0]
                return self.winner

    def winner_since(self):
        # set time when a winner win
        if self.winner:
            if self.block == 0:
                self.block = 1
                self.since = time()
        if time() - self.since > 3:
            self.block = 0
        return self.since

    def delete_disconnected_player(self):
        for addr, t in self.ACTA.items():
            if time() -t > 1:
                del self.manage[addr]
                del self.ACTA[addr]
                print "Deleted in ACTA, in manage, and I kill the player: ", addr

    def create_msg_for_all_players(self):
        # si manage est vide, le message est
        # /for_all_players ,s s:classement
        self.msg = osc.Message("/for_all_players")
        for key, value in self.manage.items():
            self.msg.add(key[0])
            for item in value[2]:
                self.msg.add(item)
        self.msg.add("classement")
        for value in self.manage.values():
            self.msg.add(value[0])
            self.msg.add(value[1])
        return self.msg

    def reset_data(self):
        self.manage = {}
        #self.ACTA = {}
        print "Reset in Game Dictionnary"

    def print_some(self):
        for v in self.manage.values(): print v[0], v[1]

if __name__ == "__main__":
    # only to test
    # set instance of class
    game = GameManagement()

    # s:192.168.1.7  s:/ball f:7.8 f:8.9 s:/bat f:7.1 f:0.0 s:/score i:97 s:/name s:hhhh

    # only to test this script
    test = [
            [('10.0.0.7', 47582), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 4, '/name',  'toto']],
            [('10.0.0.5', 41258), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 3, '/name',  'lui' ]],
            [('10.0.0.1', 65872), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 2, '/name',  'qui' ]],

            [('10.0.0.7', 47582), ['/ball', 3.7, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'toto' ]],
            [('10.0.0.5', 41258), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 2, '/name',  'lui' ]],
            [('10.0.0.1', 65872), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 1, '/name',  'qui' ]],

            [('10.0.0.7', 47582), ['/ball', 3.8, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'toto' ]],
            [('10.0.0.5', 41258), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'lui' ]],
            [('10.0.0.1', 65872), ['/ball', 3.6, 0.0, '/bat', 9.5, 0.0, '/score', 1, '/name',  'qui' ]],

            [('10.0.0.7', 47582), ['/ball', 3.9, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'toto' ]],
            [('10.0.0.5', 41258), ['/ball', 3.8, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'lui' ]],
            [('10.0.0.1', 65872), ['/ball', 3.1, 0.0, '/bat', 9.5, 0.0, '/score', 1, '/name',  'qui' ]],

            [('10.0.0.7', 47582), ['/ball', 3.9, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'toto' ]],
            [('10.0.0.5', 41258), ['/ball', 3.8, 0.0, '/bat', 9.5, 0.0, '/score', 0, '/name',  'lui' ]],
            [('10.0.0.1', 65872), ['/ball', 3.1, 0.0, '/bat', 9.5, 0.0, '/score', 1, '/name',  'qui' ]]
            ]
            #    address             message
    for t in test:
        game.insert_data_in_dict(t[1], t[0]) # message, address
    game.print_some()
    game.who_is_the_winner()



